"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path = require("path");
var webpack = require("webpack");
var node_fetch_1 = require("node-fetch");
var express = require("express");
var middleware = require("webpack-dev-middleware");
var httpProxy = require("http-proxy");
var http = require("http");
var compression = require("compression");
var requireFromString = require("require-from-string");
var sourceMap = new Map();
require('source-map-support').install({
    retrieveSourceMap: function (url) {
        var map = sourceMap.get(url);
        if (map) {
            return {
                url: url,
                map: map.toString(),
            };
        }
        return null;
    }
});
var AssetReadError = /** @class */ (function (_super) {
    tslib_1.__extends(AssetReadError, _super);
    function AssetReadError(response) {
        var _this = _super.call(this, 'Response read error.') || this;
        _this.response = response;
        return _this;
    }
    AssetReadError.code = 'ASSET_READ_ERROR';
    return AssetReadError;
}(Error));
exports.AssetReadError = AssetReadError;
function outputFile(stats) {
    var _a = stats.toJson({ all: false, entrypoints: true, outputPath: true }), entrypoints = _a.entrypoints, outputPath = _a.outputPath;
    var main;
    for (var _i = 0, _b = Object.keys(entrypoints); _i < _b.length; _i++) {
        var key = _b[_i];
        for (var _c = 0, _d = entrypoints[key].assets; _c < _d.length; _c++) {
            var asset = _d[_c];
            if (asset.endsWith('.js')) {
                main = asset;
                break;
            }
        }
        if (main) {
            break;
        }
    }
    return path.join(outputPath, main);
}
function watch(config, host, port, webpackDevserverUrl) {
    if (host === void 0) { host = '0.0.0.0'; }
    if (port === void 0) { port = 5000; }
    if (webpackDevserverUrl === void 0) { webpackDevserverUrl = 'http://localhost:3000/'; }
    return new Promise(function (resolve, reject) {
        var lastEndTime;
        var serverMiddleware;
        var app = express();
        var proxy = httpProxy.createProxyServer({
            target: webpackDevserverUrl,
            ws: true,
        });
        app.use(compression());
        app.use(middleware(webpack(config), {
            logLevel: 'info',
            serverSideRender: true,
            stats: 'minimal',
        }));
        app.use(function (req, res, next) {
            var _a = res.locals, fs = _a.fs, webpackStats = _a.webpackStats;
            res.locals.readAsset = function (asset) {
                return node_fetch_1.default("" + webpackDevserverUrl + asset)
                    .then(function (response) {
                    if (response.ok) {
                        if (asset.endsWith('.json')) {
                            return response.json();
                        }
                        return response.text();
                    }
                    return Promise.reject(new AssetReadError(response));
                });
            };
            if (lastEndTime !== webpackStats.endTime) {
                var filename = outputFile(webpackStats);
                var mapFilename = filename + ".map";
                if (fs.existsSync(mapFilename)) {
                    sourceMap.set(filename, fs.readFileSync(mapFilename));
                }
                serverMiddleware = requireFromString(fs.readFileSync(filename, 'utf-8'), filename).default;
            }
            lastEndTime = webpackStats.endTime;
            serverMiddleware(req, res, next);
        });
        app.use(function (req, res, next) {
            proxy.web(req, res, {}, next);
        });
        var server = http.createServer(app);
        server.on('upgrade', function (req, socket, head) {
            proxy.ws(req, socket, head);
        });
        server.listen(port, host, function (err) {
            if (err) {
                reject(err);
                return;
            }
            resolve();
        });
    });
}
exports.watch = watch;
//# sourceMappingURL=index.js.map