import * as tslib_1 from "tslib";
import * as path from 'path';
import { runInThisContext } from 'vm';
import * as webpack from 'webpack';
import fetch from 'node-fetch';
import * as express from 'express';
import * as middleware from 'webpack-dev-middleware';
import * as httpProxy from 'http-proxy';
import * as http from 'http';
import * as compression from 'compression';
var sourceMap = new Map();
require('source-map-support').install({
    retrieveSourceMap: function (url) {
        var map = sourceMap.get(url);
        if (map) {
            return {
                url: url,
                map: map.toString(),
            };
        }
        return null;
    }
});
var AssetReadError = /** @class */ (function (_super) {
    tslib_1.__extends(AssetReadError, _super);
    function AssetReadError(response) {
        var _this = _super.call(this, 'Response read error.') || this;
        _this.response = response;
        return _this;
    }
    AssetReadError.code = 'ASSET_READ_ERROR';
    return AssetReadError;
}(Error));
export { AssetReadError };
function outputFile(stats) {
    var _a = stats.toJson({ all: false, entrypoints: true, outputPath: true }), entrypoints = _a.entrypoints, outputPath = _a.outputPath;
    var main;
    for (var _i = 0, _b = Object.keys(entrypoints); _i < _b.length; _i++) {
        var key = _b[_i];
        for (var _c = 0, _d = entrypoints[key].assets; _c < _d.length; _c++) {
            var asset = _d[_c];
            if (asset.endsWith('.js')) {
                main = asset;
                break;
            }
        }
        if (main) {
            break;
        }
    }
    return path.join(outputPath, main);
}
export function watch(config, host, port, webpackDevserverUrl) {
    if (host === void 0) { host = '0.0.0.0'; }
    if (port === void 0) { port = 5000; }
    if (webpackDevserverUrl === void 0) { webpackDevserverUrl = 'http://localhost:3000/'; }
    return new Promise(function (resolve, reject) {
        var lastEndTime;
        var serverMiddleware;
        var app = express();
        var proxy = httpProxy.createProxyServer({
            target: webpackDevserverUrl,
            ws: true,
        });
        app.use(compression());
        app.use(middleware(webpack(config), {
            logLevel: 'info',
            serverSideRender: true,
            stats: 'minimal',
        }));
        app.use(function (req, res, next) {
            var _a = res.locals, fs = _a.fs, webpackStats = _a.webpackStats;
            res.locals.readAsset = function (asset) {
                return fetch("" + webpackDevserverUrl + asset).then(function (response) {
                    if (response.ok) {
                        if (asset.endsWith('.json')) {
                            return response.json();
                        }
                        return response.text();
                    }
                    return Promise.reject(new AssetReadError(response));
                });
            };
            if (lastEndTime !== webpackStats.endTime) {
                var filename = outputFile(webpackStats);
                var mapFilename = filename + ".map";
                if (fs.existsSync(mapFilename)) {
                    sourceMap.set(filename, fs.readFileSync(mapFilename));
                }
                serverMiddleware = runInThisContext(fs.readFileSync(filename), filename).default;
            }
            lastEndTime = webpackStats.endTime;
            serverMiddleware(req, res, next);
        });
        app.use(function (req, res) {
            proxy.web(req, res);
        });
        var server = http.createServer(app);
        server.on('upgrade', function (req, socket, head) {
            proxy.ws(req, socket, head);
        });
        server.listen(port, host, function (err) {
            if (err) {
                reject(err);
                return;
            }
            resolve();
        });
    });
}
